@@ -1,5 +1,7 @@
 from __future__ import annotations
 
+import argparse
+import importlib
 import json
 
 # Inlined minimal helpers from x_make_common_x.helpers
@@ -11,18 +13,30 @@ import sys
 import sys as _sys
 import urllib.request
 import uuid
+from collections.abc import Mapping, Sequence
 from contextlib import contextmanager, suppress
+from dataclasses import dataclass
 from pathlib import Path
+from types import MappingProxyType, SimpleNamespace
 from typing import IO, TYPE_CHECKING, TypeVar, cast
 from urllib.error import HTTPError, URLError
 from urllib.parse import urlparse
 
 if TYPE_CHECKING:
-    from collections.abc import Iterable, Iterator, Mapping
+    from collections.abc import Iterable, Iterator
+
+from jsonschema import ValidationError
+
+from x_0_make_all_x.manifest import ManifestEntry, ManifestOptions
+from x_make_common_x.json_contracts import validate_payload
+from x_make_pypi_x.json_contracts import ERROR_SCHEMA, INPUT_SCHEMA, OUTPUT_SCHEMA
+from x_make_pypi_x.publish_flow import PublisherFactory, publish_manifest_entries
 
 _LOGGER = logging.getLogger("x_make")
 _T = TypeVar("_T")
 
+_EMPTY_MAPPING: Mapping[str, object] = MappingProxyType({})
+
 
 def _info(*args: object) -> None:
     msg = " ".join(str(a) for a in args)
@@ -64,9 +78,10 @@ def _ctx_flag(ctx: object | None, attr: str, *, default: bool = False) -> bool:
     if ctx is None:
         return default
     try:
-        value = cast("object", getattr(ctx, attr))
+        value_obj: object = getattr(ctx, attr)
     except (AttributeError, RuntimeError, ValueError, TypeError):
         return default
+    value = value_obj
     if isinstance(value, bool):
         return value
     if isinstance(value, str):
@@ -420,7 +435,7 @@ class XClsMakePypiX(BaseMake):
                 message = f"Ancillary file '{ancillary_file}' is not found."
                 raise FileNotFoundError(message)
 
-    def publish(  # noqa: C901, PLR0912
+    def publish(  # noqa: C901, PLR0912, PLR0915
         self,
         main_file: str,
         ancillary_files: list[str],
@@ -486,6 +501,20 @@ class XClsMakePypiX(BaseMake):
             )
 
         skip_existing = self.get_env_bool("TWINE_SKIP_EXISTING", default=True)
+        env_snapshot = {
+            "TWINE_USERNAME": bool(os.environ.get("TWINE_USERNAME")),
+            "TWINE_PASSWORD": bool(os.environ.get("TWINE_PASSWORD")),
+            "TWINE_API_TOKEN": bool(os.environ.get("TWINE_API_TOKEN")),
+        }
+        snapshot_parts = [
+            f"{name}={'set' if present else 'unset'}"
+            for name, present in env_snapshot.items()
+        ]
+        _info(
+            "Twine environment snapshot (pre-upload):",
+            "; ".join(snapshot_parts),
+        )
+        _info("Twine publisher module path:", __file__)
         base_cmd = [sys.executable, "-m", "twine", "upload"]
         if skip_existing:
             base_cmd.append("--skip-existing")
@@ -510,9 +539,325 @@ class XClsMakePypiX(BaseMake):
         self.publish(main_file, ancillary_files or [])
 
 
-if __name__ == "__main__":
-    message = "This file is not meant to be run directly."
-    raise SystemExit(message)
+def _failure_payload(
+    message: str, *, details: Mapping[str, object] | None = None
+) -> dict[str, object]:
+    payload: dict[str, object] = {"status": "failure", "message": message}
+    if details:
+        payload["details"] = {str(key): value for key, value in details.items()}
+    with suppress(ValidationError):
+        validate_payload(payload, ERROR_SCHEMA)
+    return payload
+
+
+def _normalize_string(value: object) -> str | None:
+    if isinstance(value, str):
+        text = value.strip()
+        return text or None
+    return None
+
+
+def _normalize_string_list(values: object) -> tuple[str, ...]:
+    if isinstance(values, Sequence) and not isinstance(values, (str, bytes, bytearray)):
+        collected: list[str] = []
+        for item in values:
+            if isinstance(item, str):
+                text = item.strip()
+                if text:
+                    collected.append(text)
+        return tuple(collected)
+    return ()
+
+
+def _mapping_from_object(raw: object) -> Mapping[str, object] | None:
+    if isinstance(raw, Mapping):
+        return MappingProxyType({str(key): value for key, value in raw.items()})
+    return None
+
+
+def _options_from_json(raw: Mapping[str, object] | None) -> ManifestOptions:
+    if raw is None:
+        return ManifestOptions()
+    dependencies = _normalize_string_list(raw.get("dependencies"))
+    allowlist = _normalize_string_list(raw.get("ancillary_allowlist"))
+    ancillary_list = _normalize_string_list(raw.get("ancillary_list"))
+    extra_dict = _mapping_from_object(raw.get("extra"))
+    extra_proxy = (
+        MappingProxyType(dict(extra_dict)) if extra_dict else MappingProxyType({})
+    )
+    return ManifestOptions(
+        author=_normalize_string(raw.get("author")),
+        email=_normalize_string(raw.get("email")),
+        description=_normalize_string(raw.get("description")),
+        license_text=_normalize_string(raw.get("license_text")),
+        dependencies=dependencies,
+        pypi_name=_normalize_string(raw.get("pypi_name")),
+        ancillary_allowlist=allowlist,
+        ancillary_list=ancillary_list,
+        extra=extra_proxy,
+    )
+
+
+def _entry_from_json(entry: Mapping[str, object]) -> ManifestEntry:
+    options_raw = entry.get("options")
+    options_mapping = (
+        _mapping_from_object(options_raw) if isinstance(options_raw, Mapping) else None
+    )
+    options = _options_from_json(options_mapping)
+    ancillary = _normalize_string_list(entry.get("ancillary"))
+    package = _normalize_string(entry.get("package"))
+    version = _normalize_string(entry.get("version"))
+    if package is None or version is None:
+        message = "manifest entry is missing required fields"
+        raise ValueError(message)
+    return ManifestEntry(
+        package=package,
+        version=version,
+        ancillary=ancillary,
+        options=options,
+    )
+
+
+def _resolve_publisher_factory(identifier: str | None) -> PublisherFactory:
+    default_factory = cast("PublisherFactory", XClsMakePypiX)
+    if identifier is None:
+        return default_factory
+    cleaned = identifier.strip()
+    if not cleaned:
+        return default_factory
+    if cleaned in {"XClsMakePypiX", "x_cls_make_pypi_x"}:
+        return default_factory
+    module_name: str
+    attr_name: str
+    if ":" in cleaned:
+        module_name, attr_name = cleaned.split(":", 1)
+    elif "." in cleaned:
+        module_name, attr_name = cleaned.rsplit(".", 1)
+    else:
+        module_name, attr_name = "x_make_pypi_x.x_cls_make_pypi_x", cleaned
+    module = importlib.import_module(module_name)
+    candidate = cast("object", getattr(module, attr_name))
+    if not callable(candidate):
+        message = f"publisher_factory '{identifier}' did not resolve to a callable"
+        raise TypeError(message)
+    return cast("PublisherFactory", candidate)
+
+
+def _build_context(
+    ctx: object | None, overrides: Mapping[str, object] | None
+) -> object | None:
+    if not overrides:
+        return ctx
+    namespace = SimpleNamespace(**{str(key): value for key, value in overrides.items()})
+    if ctx is not None:
+        namespace._parent_ctx = ctx  # noqa: SLF001 - preserve legacy attribute
+    return namespace
+
+
+def _validate_input_schema(payload: Mapping[str, object]) -> dict[str, object] | None:
+    try:
+        validate_payload(payload, INPUT_SCHEMA)
+    except ValidationError as exc:
+        return _failure_payload(
+            "input payload failed validation",
+            details={
+                "error": exc.message,
+                "path": [str(part) for part in exc.path],
+                "schema_path": [str(part) for part in exc.schema_path],
+            },
+        )
+    return None
+
+
+def _parameters_from_payload(payload: Mapping[str, object]) -> Mapping[str, object]:
+    parameters_obj = payload.get("parameters")
+    if isinstance(parameters_obj, Mapping):
+        return MappingProxyType(
+            {str(key): value for key, value in parameters_obj.items()}
+        )
+    return _EMPTY_MAPPING
+
+
+@dataclass(frozen=True)
+class _InputParameters:
+    entries: tuple[ManifestEntry, ...]
+    repo_root: str
+    token_env: str
+    publisher_identifier: str | None
+    context_overrides: Mapping[str, object] | None
+
+
+def _entries_from_parameters(
+    parameters: Mapping[str, object],
+) -> tuple[ManifestEntry, ...]:
+    entries_raw = parameters.get("entries")
+    manifest_entries: list[ManifestEntry] = []
+    if isinstance(entries_raw, Sequence) and not isinstance(
+        entries_raw, (str, bytes, bytearray)
+    ):
+        manifest_entries.extend(
+            _entry_from_json(entry_obj)
+            for entry_obj in entries_raw
+            if isinstance(entry_obj, Mapping)
+        )
+    return tuple(manifest_entries)
+
+
+def _extract_inputs(
+    parameters: Mapping[str, object],
+) -> _InputParameters | dict[str, object]:
+    try:
+        manifest_entries = _entries_from_parameters(parameters)
+    except ValueError as exc:
+        return _failure_payload(
+            "failed to parse manifest entries",
+            details={"error": str(exc)},
+        )
+
+    repo_root = _normalize_string(parameters.get("repo_parent_root"))
+    if repo_root is None:
+        return _failure_payload(
+            "repo_parent_root must be a non-empty string",
+            details={"field": "repo_parent_root"},
+        )
+
+    token_env = _normalize_string(parameters.get("token_env"))
+    token_env_effective = token_env or XClsMakePypiX.TEST_PYPI_TOKEN_ENV
+
+    publisher_identifier = _normalize_string(parameters.get("publisher_factory"))
+
+    context_raw = parameters.get("context")
+    context_overrides = _mapping_from_object(context_raw)
+
+    return _InputParameters(
+        entries=manifest_entries,
+        repo_root=repo_root,
+        token_env=token_env_effective,
+        publisher_identifier=publisher_identifier,
+        context_overrides=context_overrides,
+    )
+
+
+def main_json(  # noqa: PLR0911 - routing handles multiple exit paths explicitly
+    payload: Mapping[str, object], *, ctx: object | None = None
+) -> dict[str, object]:
+    schema_failure = _validate_input_schema(payload)
+    if schema_failure:
+        return schema_failure
+
+    parameters = _parameters_from_payload(payload)
+    inputs = _extract_inputs(parameters)
+    if isinstance(inputs, dict):
+        return inputs
+
+    try:
+        publisher_factory = _resolve_publisher_factory(inputs.publisher_identifier)
+    except Exception as exc:  # noqa: BLE001
+        return _failure_payload(
+            "failed to resolve publisher factory",
+            details={"error": str(exc)},
+        )
+
+    runtime_ctx = _build_context(ctx, inputs.context_overrides)
+
+    cloner = SimpleNamespace(target_dir=inputs.repo_root)
+
+    try:
+        _, _, report_path = publish_manifest_entries(
+            inputs.entries,
+            cloner=cloner,
+            ctx=runtime_ctx,
+            repo_parent_root=inputs.repo_root,
+            publisher_factory=publisher_factory,
+            token_env=inputs.token_env,
+        )
+    except Exception as exc:  # noqa: BLE001
+        details: dict[str, object] = {"error": str(exc)}
+        report_path_obj = cast("object | None", getattr(exc, "run_report_path", None))
+        if isinstance(report_path_obj, Path):
+            details["run_report_path"] = str(report_path_obj)
+        return _failure_payload("publishing manifest entries failed", details=details)
+
+    try:
+        report_text = Path(report_path).read_text(encoding="utf-8")
+        result_payload_raw: object = json.loads(report_text)
+    except (OSError, json.JSONDecodeError) as exc:
+        return _failure_payload(
+            "failed to read run report",
+            details={
+                "error": str(exc),
+                "report_path": str(report_path),
+            },
+        )
+
+    if not isinstance(result_payload_raw, Mapping):
+        return _failure_payload(
+            "run report did not contain a JSON object",
+            details={"report_path": str(report_path)},
+        )
+
+    typed_result_mapping = cast("Mapping[str, object]", result_payload_raw)
+    result_payload = dict(typed_result_mapping.items())
+    errors_obj = result_payload.get("errors")
+    if not isinstance(errors_obj, list):
+        result_payload["errors"] = cast("list[object]", [])
+
+    outputs: dict[str, object] = dict(result_payload)
+    try:
+        validate_payload(outputs, OUTPUT_SCHEMA)
+    except ValidationError as exc:
+        return _failure_payload(
+            "generated output failed schema validation",
+            details={
+                "error": exc.message,
+                "path": [str(part) for part in exc.path],
+                "schema_path": [str(part) for part in exc.schema_path],
+            },
+        )
+    return outputs
+
+
+def _load_json_payload(file_path: str | None) -> Mapping[str, object]:
+    def _load(stream: IO[str]) -> Mapping[str, object]:
+        payload_obj: object = json.load(stream)
+        if not isinstance(payload_obj, Mapping):
+            message = "JSON payload must be a mapping"
+            raise TypeError(message)
+        payload_mapping = cast("Mapping[str, object]", payload_obj)
+        typed_payload = {str(key): value for key, value in payload_mapping.items()}
+        return MappingProxyType(typed_payload)
+
+    if file_path:
+        with Path(file_path).open("r", encoding="utf-8") as handle:
+            return _load(handle)
+    return _load(sys.stdin)
+
+
+def _run_json_cli(args: Sequence[str]) -> None:
+    parser = argparse.ArgumentParser(description="x_make_pypi_x JSON runner")
+    parser.add_argument(
+        "--json", action="store_true", help="Read JSON payload from stdin"
+    )
+    parser.add_argument("--json-file", type=str, help="Path to JSON payload file")
+    parsed = parser.parse_args(args)
+
+    json_flag_source: object = getattr(parsed, "json", False)
+    json_flag = bool(json_flag_source)
+    json_file_arg = cast("str | None", getattr(parsed, "json_file", None))
+
+    if not (json_flag or json_file_arg):
+        parser.error("JSON input required. Use --json for stdin or --json-file <path>.")
+
+    payload = _load_json_payload(json_file_arg if json_file_arg else None)
+    result = main_json(payload)
+    json.dump(result, sys.stdout, indent=2)
+    sys.stdout.write("\n")
 
 
 x_cls_make_pypi_x = XClsMakePypiX
+
+__all__ = ["XClsMakePypiX", "main_json", "x_cls_make_pypi_x"]
+
+
+if __name__ == "__main__":
+    _run_json_cli(sys.argv[1:])
